% pb_mindrr - Design for pencil beam array with minimum DRR and specified beam efficiency.
%
% Function call:
%    [a_opt,DRR_opt,BE_opt]=pb_mindrr(N,BW,BE_des,e_tol)
% where
%    N           - double scalar, number of array elements
%    BW          - double scalar, width of pencil beam, in rad
%    BE_des      - double scalar, specified minimum beam efficiency
%    e_tol       - double scalar, tolerance for DRR
%    a_opt       - double vector, optimum coefficients
%    DRR_opt     - double scalar, optimum dynamic range ratio
%    BE_opt      - double scalar, obtained beam efficiency
%
% Algorithm:
%    M. Matijascic, M. Jurisic Bellotti, M. Vucic, G. Molnar,
%    Optimum Synthesis of Pencil Beams with Constrained Dynamic Range Ratio
%    International Journal of Antennas and Propagation, 2022
%    Section 6
%
% Notes:
%    1. Interelement spacing of lambda/2 is used.
%    2. Global search is used in Step 2 (for calculating a_1) and in Step 8.
%       If all positive coefficients are preferred, do the following
%           - comment lines:   ... % global, arbitrary coefficients 
%           - uncomment lines: ... % convex, all-positive coefficients
%

function [a_opt,DRR_opt,BE_opt]=pb_mindrr(N,BW,BE_des,e_tol)

d=1/2;
lambda=1;

% For specified N and BW calculate
%   coefficients a_dpss of DPSS array 
W=1/lambda*d*sin(BW/2);
[E,~]=dpss(N,N*W);
a_dpss=E(:,1)/sum(E(:,1));
%   coefficients a_1 of uniform excitation array 
a_1=pb_global(d*(0:N-1),1,lambda,BW,N*10+1,'SLP'); % global, arbitrary coefficients
% a_1=pb_positive(d*(0:N-1),1,lambda,BW,N*10+1,'SLP'); % convex, all-positive coefficients

% Calculate
%  BE_high as beam efficiency of array with coefficients a_dpss
BE_high=pb_be(d*(0:N-1),a_dpss,lambda,BW);
%  BE_low as beam efficiency of array with coefficients a_1
BE_low=pb_be(d*(0:N-1),a_1,lambda,BW);

if(BE_des>BE_high)
   error('BE_des is too large for fiven N and BW.');
end

if(BE_des<BE_low)
   error('BE_des is too small for given N and BW.');
end

% Calculate DRR_high as dynamic range ratio of a_dpss
DRR_high=max(abs(a_dpss))/min(abs(a_dpss));

% Set DRR_low, dynamic range ratio of a_1
DRR_low=1;

while (DRR_high-DRR_low)>e_tol
   D=(DRR_low+DRR_high)/2;
   a=pb_global(d*(0:N-1),D,lambda,BW,N*10+1,'SLP'); % global, arbitrary coefficients
   % a=pb_positive(d*(0:N-1),D,lambda,BW,N*10+1,'SLP'); % convex, all-positive coefficients

   BE=pb_be(d*(0:N-1),a,lambda,BW);
   if BE<BE_des
      DRR_low=D;
   else
      DRR_high=D;
   end
end

a_opt=a;
DRR_opt=D;
BE_opt=BE;


