% pb_positive - Global design of linear pencil beams with constrained DRR.
%
% Function call:
%    [a_opt,d_opt]=pb_positive(x,D,lambda,BW,Q,DesignType)
% where
%    x           - double vector, absolute positions of arrays coefficients in meters
%    D           - double scalar, upper bound of DRR
%    lambda      - double scalar, wavelength at operating frequency
%    BW          - double scalar, width of pencil beam in radians
%    Q           - double scalar, even number of points in care band
%                     Suggested values for arrays with N elements:
%                        Q=10*N   if DesignType='SLL'
%                        Q=10*N+1 if DesignType='SLP'
%    DesignType  - string, type of approximation
%                     'SLL' - minimization of sidelobe level
%                     'SLP' - minimization of sidelobe power
%    a_opt       - double vector, optimum coefficients
%    d_opt       - double scalar, objective function at a_opt
%
% Algorithm:
%    M. Matijascic, M. Jurisic Bellotti, M. Vucic, G. Molnar,
%    Optimum Synthesis of Pencil Beams with Constrained Dynamic Range Ratio
%    International Journal of Antennas and Propagation, 2022
%    Section 2
%

function [a_opt,d_opt]=pb_positive(x,D,lambda,BW,Q,DesignType)

% ==========================================================================
%                          PARAMETER CHECK
% ==========================================================================

if strcmp(DesignType,'SLL')
elseif strcmp(DesignType,'SLP')
   if mod(Q,2)==0
      error('In minimization of SLP, parameter Q should be odd.');
   end
else
   error('Illegal parameter DesignType.')
end

% ==========================================================================
%                           PREPROCESSING
% ==========================================================================

x=x(:);
N=length(x);
S=ones(1,N);

uq=2*pi/lambda*linspace(sin(BW/2),1,Q);

if strcmp(DesignType,'SLP')
   dx=(1-sin(BW/2))/((Q-1)*3);
   mu=1/sqrt(4*pi*dx); % factor in SOC constraint
end

% ==========================================================================
%                             SOLVER CALL
% ==========================================================================

if strcmp(DesignType,'SLL')
   [a_opt,d_opt]=pb14(x,S,D,uq);
elseif strcmp(DesignType,'SLP')
   [a_opt,d_opt]=pb22(x,S,D,uq,mu);
end

